# -*- encoding: utf-8 -*-

from django.contrib.auth.models import User
from models import UserProfile

import logging
import facebook

logger = logging.getLogger(__name__)

class FacebookBackend:
    
    """
        Autentificacion basada en un token
    """
    def authenticate(self, token=None):        
        
        graph = facebook.GraphAPI(token)
    
        profile = graph.get_object("me")
        logger.debug(profile)
        uid = profile["id"]
           
        """
            El nombre de nuestro usuario sera el id de facebook
            No seria mejor tener un UserProfile para facebook que aniadiese un fb_id
            y rellenase los campos mejor
        """
        
        
        try:
            userProfile = UserProfile.objects.get(uid=uid)
            userProfile.access_token = token
            """
            TODO: falta por actualizar la fecha de expiración del token.
            """
            
            
            userProfile.save()
        
            logger.debug("Encontrado userProfile: %s" % userProfile.user.username)
            return userProfile.user
        
        except UserProfile.DoesNotExist:
            user = User()
            user.set_unusable_password()
            user.email = profile['email']
            user.username = uid
            user.first_name = profile['first_name']
            user.last_name = profile['last_name']
            user.save()
            
            
            userProfile = UserProfile()
            userProfile.user = user;
            userProfile.access_token = token
            userProfile.uid = profile["id"]
            
            userProfile.save()
            logger.debug("-NO- Encontrado userProfile: %s" % user)
            return user
            

    
        """
            Esto no haria falta hacerlo cada vez,no?
            Solo cuando se crea el usuario
        """
        
        """
        user.set_unusable_password()
        user.email = profile['email']
        user.first_name = profile['first_name']
        user.last_name = profile['last_name']
        user.save()
        """
        
        
        
        """
            En esta parte hemos creado el usuario en nuestro sistema si no existia
            o bien hemos conseguido el usuario de nuestro sistema si existtia
        """

        """
            Borramos la session?
        """
        
        """
        try:
            FacebookSession.objects.get(uid=profile['id']).delete()
        except FacebookSession.DoesNotExist, e:
            pass

        facebook_session.uid = profile['id']
        facebook_session.user = user
        facebook_session.save()
        """

        return user
   
    def get_user(self, user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None
        


# Create your views here.

from django.shortcuts import render_to_response
from django.template.context import RequestContext
from django.views.decorators.csrf import csrf_exempt
from django.http import HttpResponseRedirect

from django.conf import settings
from django.contrib import auth


from facebook import get_access_token_from_code, GraphAPI

import logging
from django_facebook_helper.forms import WallPostForm
logger = logging.getLogger(__name__)



"""
friends = graph.get_connections("me", "friends")
params["friends"] = friends

params["get"] = request.GET

print params

graph.put_object("me", "feed", message="Hello, world")
""" 

def logout(request):
    auth.logout(request)
    return HttpResponseRedirect("http://local.filmsobsession.com:8000")


@csrf_exempt
def login_redirect(request):
    try:
        code = request.GET["code"]
        access_token_from_code = get_access_token_from_code(code, settings.FACEBOOK_REDIRECT_URL, settings.FACEBOOK_APP_ID, settings.FACEBOOK_APP_SECRET)
        
        access_token = access_token_from_code["access_token"]
        expires = access_token_from_code["expires"]
        
        logger.debug("access_token: %s" % access_token)
        logger.debug("expires: %s" % expires)
        
        user = auth.authenticate(token=access_token)
        logger.debug("user: %s" % user)
        
        if user is not None:
            if user.is_active:
                auth.login(request, user) # Redirect to a success page.
            else:
                pass # Return a 'disabled account' error message
        else:
            pass # Return an 'invalid login' error message.
    except:
        pass 
    
    return render_to_response('facebook-redirect.html', {}, context_instance=RequestContext(request)) 
         
   
def friends(request):
    access_token = request.user.profile.access_token
    logger.debug("access_token: %s" % access_token) 
    
    graph = GraphAPI(access_token = access_token)
    user_friends = graph.get_connections("me", "friends")

    logger.debug("friends: %s" % user_friends) 
    logger.debug("friends: %s" % user_friends["data"]) 

    return render_to_response('friends.html', {"friends" : user_friends["data"]}, context_instance=RequestContext(request))   
        
     
def wall_post(request):
    
    if request.method == 'POST': # If the form has been submitted...
        form = WallPostForm(request.POST) # A form bound to the POST data
        if form.is_valid(): # All validation rules pass
            # Process the data in form.cleaned_data
            message = form.cleaned_data['message']
            
            access_token = request.user.profile.access_token   
            logger.debug("access_token: %s" % access_token) 
            
            graph = GraphAPI(access_token = access_token)
            user_friends = graph.put_wall_post(message)
    
    
            return HttpResponseRedirect('/') # Redirect after POST
    else:
        form = WallPostForm() # An unbound form

    
    return render_to_response('wall-post.html', {"form" : form}, context_instance=RequestContext(request))   
     
    
    
    
    
    
    
        
        
        
    
    


from django.db import models
from django.contrib.auth.models import User




class UserProfile(models.Model):
    user = models.ForeignKey(User, unique=True)
    access_token = models.CharField(max_length=103, unique=True)
    expires = models.IntegerField(null=True)
    uid = models.BigIntegerField(unique=True, null=True)
        
    class Meta:
        unique_together = ('user', 'uid')
        
    def __unicode__(self):
        return u'Session for %s ' % self.email


User.profile = property(lambda u: UserProfile.objects.get_or_create(user=u)[0])